
version 6.0

# delimit;

capture log close;

log using M:\Distn\choicesetpaper\2004choicesets\choiceset_3mo_17pct_63pct_1co.log, replace;

/*choiceset_3mo_17pct_63pct_1co.do:  get choice sets from mergemaster1 for clogits.
Time window of 3 months; budget min of 17% and constraint of 63% of adjinc; same county.
40,000 households with 15 choices.*/

clear;

set mem 100000;

set more off;

set seed 123456789;

use M:\Distn\newpubgood\Data\Pooled\mergemaster3;

/*do some additional data cleaning & dropping of vars*/

drop if SALEPRCE<50000; /*still some very low sale prices after trim*/
gen x=mod(bathrms,0.25); /*some weird bathrm values*/
drop if x~=0 & x~=.;
drop x;

/*drop obs with missing values now.  No point in spending lots of 
computer time to draw obs that will be dropped later.*/

drop if SALEPRCE==.;
drop if bathrms==.;
drop if bedrms==.;
drop if sqftbldg==.;
drop if sqftlot==.;
drop if age==.;
drop if pool==.;
drop if firepl==.;
drop if inc_ownr==.;

/*adjust incomes*/
 gen adjinc=inc_ownr;
 replace adjinc=adjinc*1.014 if SALEYEAR==1990;
 replace adjinc=adjinc*1.027 if SALEYEAR==1991;
 replace adjinc=adjinc*1.041 if SALEYEAR==1992;
 replace adjinc=adjinc*1.074 if SALEYEAR==1993;
 replace adjinc=adjinc*1.106 if SALEYEAR==1994;
 replace adjinc=adjinc+(.116*97600) if SALEYEAR==1989;
 replace adjinc=adjinc+(.116*97600) if SALEYEAR==1990;
 replace adjinc=adjinc+(.116*97600) if SALEYEAR==1991;
 replace adjinc=adjinc+(.116*97700) if SALEYEAR==1992;
 replace adjinc=adjinc+(.116*99200) if SALEYEAR==1993;
 replace adjinc=adjinc+(.116*100800) if SALEYEAR==1994;

 /*be parsimonious in this file.  Then merge variables back in later.*/
keep idno COUNTY saledat SALEPRCE adjinc;
gen     county=1 if COUNTY=="la";
replace county=2 if COUNTY=="or";
replace county=3 if COUNTY=="va";
replace county=4 if COUNTY=="sb";
replace county=5 if COUNTY=="rv";
drop COUNTY;

tempfile mother;
save "mother", replace;

clear;

/*************************************************************************/

/*define program for generating 5 choicesets*/

capture program drop chset;
program define chset;

local h=1;
while `h'<=3 {;

use "mother";
gen ranuni = uniform();

/*get data set to sample from and set of 40000 houses actualy chosen*/

replace ranuni=1 if 0.116*SALEPRCE/adjinc <= 0.17 | 0.116*SALEPRCE/adjinc >= 0.63;
sort ranuni;

gen x=0;
replace x=-1 if _n<=40000;
sort x saledat;
gen hhold = _n if _n <= 40000; /*numeric id for _household_*/
sort saledat;
gen idno2 =_n; /*need a numeric id for _house_*/
drop x;

save "sample", replace;

keep if hhold ~=.;
drop if ranuni==1; 
drop ranuni;
gen choice=1;
tempfile temp1;
save "temp1", replace;

clear;
set mem 50000;

/*now add houses NOT chosen for each hhold*/

tempfile temp2;
tempname nochs;
postfile `nochs' idno2 hhold flag using "temp2", replace;

/*go through week by week*/
local k = 1989;
while `k' < 1995 {;

/*time window for households observed to buy in week k*/
drop _all;
use "sample" if saledat >= `k'-(1/8) & saledat <= `k'+(7/365)+(1/8);

/*get codes for households observed this week*/
quietly replace hhold=. if saledat < `k' | saledat >= `k' + 7/365;
quietly sum hhold;
scalar min=r(min);
scalar max=r(max);

if max~=. {; /*non-empty week*/

quietly gen x=1; /*flag: x=1 means invalid observation*/
quietly gen ran=.;

  local j=min;
  while `j' <= max  {;  

  /*reset data*/
  quietly replace x=1;
  quietly replace ran = uniform();

  /*get info on choosing hhold*/
  quietly sum adjinc if hhold==`j', meanonly;
  scalar inc=r(mean);
  quietly sum county if hhold==`j', meanonly;
  scalar CO=r(mean);
  quietly sum saledat if hhold==`j', meanonly;
  scalar date=r(mean);

  quietly replace x=0 if hhold~=`j'
                       & 0.116*SALEPRCE/inc > 0.17
                       & 0.116*SALEPRCE/inc < 0.63
                       & saledat >= date-(1/8) & saledat <= date+(1/8)
                       & county==CO;  /*valid choices*/

  quietly sort x ran; /*random selection of valid choices at top*/

  capture {;

  local i=1; /*get top 14 obs.  Hopefully 1 valid, if not, delete below*/
  while `i' <= 14 {; 

    post `nochs' idno2[`i'] `j' x[`i'];

    local i = `i'+1;

    };

  }; /*end capture*/

  if _rc~=0 {; /*if BREAK or error, save results before exiting*/
    postclose `nochs';
    exit;
  };

  local j = `j'+1; 
   
  };

}; /*end commands if max~=.*/

local k = `k' + 7/365;

};

postclose `nochs';

/*merge with chosen houses*/

clear;

use "temp2";
drop if flag==1; /*delete any invalid choices in top 14*/
drop flag;
sort idno2;
gen choice=0;
save "temp2", replace;
clear;

use "sample";
drop hhold;
sort idno2;
merge idno2 using "temp2";
tab _merge;
keep if choice==0;
drop adjinc _merge;
append using "temp1";
drop idno2;
gsort hhold -choice;
drop ranuni;

egen x=max(adjinc), by(hhold);
replace adjinc=x;
drop x county;

/*bring back in other descriptive variables when working*/

local x="M:\Distn\choicesetpaper\2004choicesets\choiceset_3mo_17pct_63pct_1co_"+"`h'";

save `x', replace;

local h = `h'+1;

};

end;

chset;

log close;  
  

    
 






